@@ -14,8 +14,7 @@
  * limitations under the License.
  */
 
-#ifndef AIDL_AST_CPP_H_
-#define AIDL_AST_CPP_H_
+#pragma once
 
 #include <memory>
 #include <string>
@@ -38,6 +37,7 @@ class AstNode {
   AstNode() = default;
   virtual ~AstNode() = default;
   virtual void Write(CodeWriter* to) const = 0;
+  std::string ToString();
 };  // class AstNode
 
 class Declaration : public AstNode {
@@ -49,6 +49,18 @@ class Declaration : public AstNode {
   DISALLOW_COPY_AND_ASSIGN(Declaration);
 };  // class Declaration
 
+class LiteralDecl : public Declaration {
+ public:
+  explicit LiteralDecl(const std::string& expression);
+  ~LiteralDecl() = default;
+  void Write(CodeWriter* to) const override;
+
+ private:
+  const std::string expression_;
+
+  DISALLOW_COPY_AND_ASSIGN(LiteralDecl);
+};  // class LiteralDecl
+
 class ClassDecl : public Declaration {
  public:
   ClassDecl(const std::string& name,
@@ -104,7 +116,7 @@ class ArgList : public AstNode {
   explicit ArgList(const std::string& single_argument);
   explicit ArgList(const std::vector<std::string>& arg_list);
   explicit ArgList(std::vector<std::unique_ptr<AstNode>> arg_list);
-  ArgList(ArgList&& arg_list);
+  ArgList(ArgList&& arg_list) noexcept;
   virtual ~ArgList() = default;
 
   void Write(CodeWriter* to) const override;
@@ -163,6 +175,7 @@ class MethodDecl : public Declaration {
     IS_OVERRIDE = 1 << 2,
     IS_PURE_VIRTUAL = 1 << 3,
     IS_STATIC = 1 << 4,
+    IS_FINAL = 1 << 5,
   };
 
   MethodDecl(const std::string& return_type,
@@ -185,6 +198,7 @@ class MethodDecl : public Declaration {
   bool is_override_ = false;
   bool is_pure_virtual_ = false;
   bool is_static_ = true;
+  bool is_final_ = false;
 
   DISALLOW_COPY_AND_ASSIGN(MethodDecl);
 };  // class MethodDecl
@@ -380,22 +394,21 @@ class CppNamespace : public Declaration {
 class Document : public AstNode {
  public:
   Document(const std::vector<std::string>& include_list,
-           std::unique_ptr<CppNamespace> a_namespace);
+           std::vector<std::unique_ptr<Declaration>> declarations);
 
   void Write(CodeWriter* to) const override;
 
  private:
   std::vector<std::string> include_list_;
-  std::unique_ptr<CppNamespace> namespace_;
+  std::vector<std::unique_ptr<Declaration>> declarations_;
 
   DISALLOW_COPY_AND_ASSIGN(Document);
 };  // class Document
 
 class CppHeader final : public Document {
  public:
-  CppHeader(const std::string& include_guard,
-            const std::vector<std::string>& include_list,
-            std::unique_ptr<CppNamespace> a_namespace);
+  CppHeader(const std::string& include_guard, const std::vector<std::string>& include_list,
+            std::vector<std::unique_ptr<Declaration>> declarations);
   void Write(CodeWriter* to) const override;
 
  private:
@@ -407,7 +420,7 @@ class CppHeader final : public Document {
 class CppSource final : public Document {
  public:
   CppSource(const std::vector<std::string>& include_list,
-            std::unique_ptr<CppNamespace> a_namespace);
+            std::vector<std::unique_ptr<Declaration>> declarations);
 
  private:
   DISALLOW_COPY_AND_ASSIGN(CppSource);
@@ -416,5 +429,3 @@ class CppSource final : public Document {
 }  // namespace cpp
 }  // namespace aidl
 }  // namespace android
-
-#endif // AIDL_AST_CPP_H_
