@@ -14,9 +14,9 @@
  * limitations under the License.
  */
 
-#ifndef AIDL_IMPORT_RESOLVER_H_
-#define AIDL_IMPORT_RESOLVER_H_
+#pragma once
 
+#include <set>
 #include <string>
 #include <vector>
 
@@ -29,8 +29,9 @@ namespace aidl {
 
 class ImportResolver {
  public:
-  ImportResolver(const IoDelegate& io_delegate,
-                 const std::vector<std::string>& import_paths);
+  ImportResolver(const IoDelegate& io_delegate, const std::string& input_file_name,
+                 const std::set<std::string>& import_paths,
+                 const std::vector<std::string>& input_files);
   virtual ~ImportResolver() = default;
 
   // Resolve the canonical name for a class to a file that exists
@@ -39,12 +40,12 @@ class ImportResolver {
 
  private:
   const IoDelegate& io_delegate_;
+  const std::string& input_file_name_;
   std::vector<std::string> import_paths_;
+  std::vector<std::string> input_files_;
 
   DISALLOW_COPY_AND_ASSIGN(ImportResolver);
 };
 
 }  // namespace android
 }  // namespace aidl
-
-#endif // AIDL_IMPORT_RESOLVER_H_
