@@ -24,8 +24,8 @@ set -e # fail early
 
 # NOTE We can't actually run these commands, since they rely on functions added
 #      by build/envsetup.sh to the bash shell environment.
-echo "+ mmma -j32 $ANDROID_BUILD_TOP/system/tools/aidl"
-make -j32 -C $ANDROID_BUILD_TOP -f build/core/main.mk \
+echo "+ mmma $ANDROID_BUILD_TOP/system/tools/aidl"
+$ANDROID_BUILD_TOP/build/soong/soong_ui.bash --make-mode \
     MODULES-IN-system-tools-aidl
 
 set -x # print commands
