@@ -67,8 +67,9 @@ using android::aidl::tests::BnNamedCallback;
 using android::aidl::tests::BnTestService;
 using android::aidl::tests::INamedCallback;
 using android::aidl::tests::SimpleParcelable;
-using android::os::PersistableBundle;
 using android::binder::Map;
+using android::os::ParcelFileDescriptor;
+using android::os::PersistableBundle;
 
 // Standard library
 using std::map;
@@ -315,6 +316,27 @@ class NativeService : public BnTestService {
     return Status::ok();
   }
 
+  Status RepeatParcelFileDescriptor(const ParcelFileDescriptor& read,
+                                    ParcelFileDescriptor* _aidl_return) override {
+    ALOGE("Repeating parcel file descriptor");
+    _aidl_return->reset(unique_fd(dup(read.get())));
+    return Status::ok();
+  }
+
+  Status ReverseParcelFileDescriptorArray(const vector<ParcelFileDescriptor>& input,
+                                          vector<ParcelFileDescriptor>* repeated,
+                                          vector<ParcelFileDescriptor>* _aidl_return) override {
+    ALOGI("Reversing parcel descriptor array of length %zu", input.size());
+    for (const auto& item : input) {
+      repeated->push_back(ParcelFileDescriptor(unique_fd(dup(item.get()))));
+    }
+
+    for (auto i = input.rbegin(); i != input.rend(); i++) {
+      _aidl_return->push_back(ParcelFileDescriptor(unique_fd(dup(i->get()))));
+    }
+    return Status::ok();
+  }
+
   Status ThrowServiceException(int code) override {
     return Status::fromServiceSpecificError(code);
   }
@@ -399,6 +421,14 @@ class NativeService : public BnTestService {
     return ReverseArray(input, repeated, _aidl_return);
   }
 
+  Status ReverseNullableUtf8CppString(
+      const unique_ptr<vector<unique_ptr<string>>>& input,
+      unique_ptr<vector<unique_ptr<string>>>* repeated,
+      unique_ptr<vector<unique_ptr<string>>>* _aidl_return) {
+
+    return ReverseUtf8CppStringList(input, repeated, _aidl_return);
+  }
+
   Status ReverseUtf8CppStringList(
       const unique_ptr<vector<unique_ptr<::string>>>& input,
       unique_ptr<vector<unique_ptr<string>>>* repeated,
@@ -430,6 +460,27 @@ class NativeService : public BnTestService {
     return Status::ok();
   }
 
+  virtual ::android::binder::Status FillOutStructuredParcelable(
+      ::android::aidl::tests::StructuredParcelable* parcelable) {
+    parcelable->shouldBeJerry = "Jerry";
+    parcelable->shouldContainThreeFs = {parcelable->f, parcelable->f, parcelable->f};
+    return Status::ok();
+  }
+
+  Status UnimplementedMethod(int32_t /* arg */, int32_t* /* _aidl_return */) override {
+    LOG_ALWAYS_FATAL("UnimplementedMethod shouldn't be called");
+  }
+
+  android::status_t onTransact(uint32_t code, const Parcel& data, Parcel* reply,
+                               uint32_t flags) override {
+    if (code == ::android::IBinder::FIRST_CALL_TRANSACTION + 45 /* UnimplementedMethod */) {
+      // pretend that UnimplementedMethod isn't implemented by this service.
+      return android::UNKNOWN_TRANSACTION;
+    } else {
+      return BnTestService::onTransact(code, data, reply, flags);
+    }
+  }
+
  private:
   map<String16, sp<INamedCallback>> service_map_;
 };
