@@ -18,6 +18,7 @@
 
 #include <algorithm>
 #include <iostream>
+#include <memory>
 #include <vector>
 
 #include <android-base/stringprintf.h>
@@ -25,14 +26,9 @@
 
 #include "logging.h"
 
-using std::cerr;
-using std::endl;
-using std::set;
 using std::string;
-using std::unique_ptr;
 using std::vector;
 
-using android::base::Split;
 using android::base::Join;
 using android::base::StringPrintf;
 
@@ -47,32 +43,11 @@ const char kNoValidMethod[] = "";
 Type* const kNoArrayType = nullptr;
 Type* const kNoNullableType = nullptr;
 
-bool is_cpp_keyword(const std::string& str) {
-  static const std::vector<std::string> kCppKeywords{
-    "alignas", "alignof", "and", "and_eq", "asm", "auto", "bitand", "bitor",
-    "bool", "break", "case", "catch", "char", "char16_t", "char32_t", "class",
-    "compl", "concept", "const", "constexpr", "const_cast", "continue",
-    "decltype", "default", "delete", "do", "double", "dynamic_cast", "else",
-    "enum", "explicit", "export", "extern", "false", "float", "for", "friend",
-    "goto", "if", "inline", "int", "long", "mutable", "namespace", "new",
-    "noexcept", "not", "not_eq", "nullptr", "operator", "or", "or_eq",
-    "private", "protected", "public", "register", "reinterpret_cast",
-    "requires", "return", "short", "signed", "sizeof", "static",
-    "static_assert", "static_cast", "struct", "switch", "template", "this",
-    "thread_local", "throw", "true", "try", "typedef", "typeid", "typename",
-    "union", "unsigned", "using", "virtual", "void", "volatile", "wchar_t",
-    "while", "xor", "xor_eq",
-  };
-  return std::find(kCppKeywords.begin(), kCppKeywords.end(), str) !=
-      kCppKeywords.end();
-}
-
 class VoidType : public Type {
  public:
   VoidType() : Type(ValidatableType::KIND_BUILT_IN, kNoPackage, "void",
                     {}, "void", kNoValidMethod, kNoValidMethod) {}
-  virtual ~VoidType() = default;
-  bool CanBeOutParameter() const override { return false; }
+  ~VoidType() override = default;
   bool CanWriteToParcel() const override { return false; }
 };  // class VoidType
 
@@ -83,6 +58,7 @@ class CppArrayType : public Type {
             const string& underlying_aidl_type,
             const string& cpp_header,
             const string& underlying_cpp_type,
+            const string& underlying_cpp_type_nulllable,
             const string& read_method,
             const string& write_method,
             bool is_nullable,
@@ -96,12 +72,11 @@ class CppArrayType : public Type {
                  ? kNoNullableType
                  // All arrays are nullable.
                  : new CppArrayType(kind, package, underlying_aidl_type,
-                                    cpp_header, underlying_cpp_type,
+                                    cpp_header, underlying_cpp_type_nulllable,
+                                    underlying_cpp_type_nulllable,
                                     read_method, write_method, true),
              src_file_name) {}
 
-  bool CanBeOutParameter() const override { return true; }
-
  private:
   static vector<string> GetHeaders(bool is_nullable, const string& cpp_header) {
     vector<string> result = {"vector"};
@@ -138,11 +113,11 @@ class PrimitiveType : public Type {
       : Type(ValidatableType::KIND_BUILT_IN, kNoPackage, aidl_type, {header},
              cpp_type, read_method, write_method,
              new CppArrayType(ValidatableType::KIND_BUILT_IN, kNoPackage,
-                              aidl_type, header, cpp_type,
+                              aidl_type, header, cpp_type, cpp_type,
                               read_array_method, write_array_method,
                               false)) {}
 
-  virtual ~PrimitiveType() = default;
+  ~PrimitiveType() override = default;
   bool IsCppPrimitive() const override { return true; }
 
  private:
@@ -158,17 +133,23 @@ class ByteType : public Type {
       : Type(ValidatableType::KIND_BUILT_IN, kNoPackage, "byte",
              {"cstdint"}, "int8_t", "readByte", "writeByte",
              new CppArrayType(ValidatableType::KIND_BUILT_IN, kNoPackage,
-                              "byte", "cstdint", "uint8_t",
+                              "byte", "cstdint", "uint8_t", "uint8_t",
                               "readByteVector", "writeByteVector",
                               false)) {}
 
-  virtual ~ByteType() = default;
+  ~ByteType() override = default;
   bool IsCppPrimitive() const override { return true; }
 
  private:
   DISALLOW_COPY_AND_ASSIGN(ByteType);
 };  // class PrimitiveType
 
+static string GetCppHeader(const AidlDefinedType& defined_type) {
+  vector<string> name = defined_type.GetSplitPackage();
+  name.push_back(defined_type.GetName());
+  return Join(name, '/') + ".h";
+}
+
 class BinderType : public Type {
  public:
   BinderType(const AidlInterface& interface, const std::string& src_file_name)
@@ -176,21 +157,18 @@ class BinderType : public Type {
                    new BinderType(interface, src_file_name, kNoNullableType,
                                   "readNullableStrongBinder"),
                    "readStrongBinder") {}
-  virtual ~BinderType() = default;
+  ~BinderType() override = default;
 
   string WriteCast(const string& val) const override {
     return write_cast_ + "(" + val + ")";
   }
 
  private:
-  BinderType(const AidlInterface& interface,
-             const std::string& src_file_name,
-             Type* nullable_type, const std::string& read)
-      : Type(ValidatableType::KIND_GENERATED,
-             interface.GetPackage(), interface.GetName(),
-             {GetCppHeader(interface)}, GetCppName(interface),
-             read, "writeStrongBinder", kNoArrayType, nullable_type,
-             src_file_name, interface.GetLine()),
+  BinderType(const AidlInterface& interface, const std::string& src_file_name, Type* nullable_type,
+             const std::string& read)
+      : Type(ValidatableType::KIND_GENERATED, interface.GetPackage(), interface.GetName(),
+             {GetCppHeader(interface)}, GetCppName(interface), read, "writeStrongBinder",
+             kNoArrayType, nullable_type, src_file_name),
         write_cast_(GetRawCppName(interface) + "::asBinder") {}
 
   static string GetCppName(const AidlInterface& interface) {
@@ -210,27 +188,17 @@ class BinderType : public Type {
     return ret;
   }
 
-  static string GetCppHeader(const AidlInterface& interface) {
-    vector<string> name = interface.GetSplitPackage();
-    name.push_back(interface.GetName());
-    return Join(name, '/') + ".h";
-  }
-
   std::string write_cast_;
 };
 
 class NullableParcelableType : public Type {
  public:
-  NullableParcelableType(const AidlParcelable& parcelable,
+  NullableParcelableType(const AidlParcelable& parcelable, const std::string& cpp_header,
                          const std::string& src_file_name)
-      : Type(ValidatableType::KIND_PARCELABLE,
-             parcelable.GetPackage(), parcelable.GetName(),
-             {parcelable.GetCppHeader()}, GetCppName(parcelable),
-             "readParcelable", "writeNullableParcelable",
-             kNoArrayType, kNoNullableType,
-             src_file_name, parcelable.GetLine()) {}
-  virtual ~NullableParcelableType() = default;
-  bool CanBeOutParameter() const override { return true; }
+      : Type(ValidatableType::KIND_PARCELABLE, parcelable.GetPackage(), parcelable.GetName(),
+             {cpp_header}, GetCppName(parcelable), "readParcelable", "writeNullableParcelable",
+             kNoArrayType, kNoNullableType, src_file_name) {}
+  ~NullableParcelableType() override = default;
 
  private:
   static string GetCppName(const AidlParcelable& parcelable) {
@@ -241,22 +209,16 @@ class NullableParcelableType : public Type {
 
 class ParcelableType : public Type {
  public:
-  ParcelableType(const AidlParcelable& parcelable,
+  ParcelableType(const AidlParcelable& parcelable, const std::string& cpp_header,
                  const std::string& src_file_name)
-      : Type(ValidatableType::KIND_PARCELABLE,
-             parcelable.GetPackage(), parcelable.GetName(),
-             {parcelable.GetCppHeader()}, GetCppName(parcelable),
-             "readParcelable", "writeParcelable",
-             new CppArrayType(
-                 ValidatableType::KIND_PARCELABLE, parcelable.GetPackage(),
-                 parcelable.GetName(), parcelable.GetCppHeader(),
-                 GetCppName(parcelable),
-                 "readParcelableVector", "writeParcelableVector", false,
-                 src_file_name),
-             new NullableParcelableType(parcelable, src_file_name),
-             src_file_name, parcelable.GetLine()) {}
-  virtual ~ParcelableType() = default;
-  bool CanBeOutParameter() const override { return true; }
+      : Type(ValidatableType::KIND_PARCELABLE, parcelable.GetPackage(), parcelable.GetName(),
+             {cpp_header}, GetCppName(parcelable), "readParcelable", "writeParcelable",
+             new CppArrayType(ValidatableType::KIND_PARCELABLE, parcelable.GetPackage(),
+                              parcelable.GetName(), cpp_header, GetCppName(parcelable),
+                              GetCppName(parcelable), "readParcelableVector",
+                              "writeParcelableVector", false, src_file_name),
+             new NullableParcelableType(parcelable, cpp_header, src_file_name), src_file_name) {}
+  ~ParcelableType() override = default;
 
  private:
   static string GetCppName(const AidlParcelable& parcelable) {
@@ -273,8 +235,7 @@ class NullableMap : public Type {
              {"binder/Map.h", "binder/Value.h"},
              "::std::unique_ptr<::android::binder::Map>",
              "readNullableMap", "writeNullableMap") {}
-  virtual ~NullableMap() = default;
-  bool CanBeOutParameter() const override { return true; }
+  ~NullableMap() override = default;
 };
 
 
@@ -288,8 +249,7 @@ class MapType : public Type {
              "readMap", "writeMap",
              kNoArrayType,
              new NullableMap() ) {}
-  virtual ~MapType() = default;
-  bool CanBeOutParameter() const override { return true; }
+  ~MapType() override = default;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(MapType);
@@ -303,8 +263,7 @@ class NullableStringListType : public Type {
              {"utils/String16.h", "memory", "vector"},
              "::std::unique_ptr<::std::vector<std::unique_ptr<::android::String16>>>",
              "readString16Vector", "writeString16Vector") {}
-  virtual ~NullableStringListType() = default;
-  bool CanBeOutParameter() const override { return true; }
+  ~NullableStringListType() override = default;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(NullableStringListType);
@@ -319,8 +278,7 @@ class StringListType : public Type {
              "::std::vector<::android::String16>",
              "readString16Vector", "writeString16Vector",
              kNoArrayType, new NullableStringListType()) {}
-  virtual ~StringListType() = default;
-  bool CanBeOutParameter() const override { return true; }
+  ~StringListType() override = default;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(StringListType);
@@ -334,8 +292,7 @@ class NullableUtf8InCppStringListType : public Type {
              {"memory", "string", "vector"},
              "::std::unique_ptr<::std::vector<std::unique_ptr<::std::string>>>",
              "readUtf8VectorFromUtf16Vector", "writeUtf8VectorAsUtf16Vector") {}
-  virtual ~NullableUtf8InCppStringListType() = default;
-  bool CanBeOutParameter() const override { return true; }
+  ~NullableUtf8InCppStringListType() override = default;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(NullableUtf8InCppStringListType);
@@ -350,8 +307,7 @@ class Utf8InCppStringListType : public Type {
              "::std::vector<::std::string>",
              "readUtf8VectorFromUtf16Vector", "writeUtf8VectorAsUtf16Vector",
              kNoArrayType, new NullableUtf8InCppStringListType()) {}
-  virtual ~Utf8InCppStringListType() = default;
-  bool CanBeOutParameter() const override { return true; }
+  ~Utf8InCppStringListType() override = default;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(Utf8InCppStringListType);
@@ -364,8 +320,7 @@ class NullableBinderListType : public Type {
              "List<android.os.IBinder>", {"binder/IBinder.h", "vector"},
              "::std::unique_ptr<::std::vector<::android::sp<::android::IBinder>>>",
              "readStrongBinderVector", "writeStrongBinderVector") {}
-  virtual ~NullableBinderListType() = default;
-  bool CanBeOutParameter() const override { return true; }
+  ~NullableBinderListType() override = default;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(NullableBinderListType);
@@ -379,8 +334,7 @@ class BinderListType : public Type {
              "::std::vector<::android::sp<::android::IBinder>>",
              "readStrongBinderVector", "writeStrongBinderVector",
              kNoArrayType, new NullableBinderListType()) {}
-  virtual ~BinderListType() = default;
-  bool CanBeOutParameter() const override { return true; }
+  ~BinderListType() override = default;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(BinderListType);
@@ -411,48 +365,37 @@ Type::Type(int kind,
 bool Type::CanWriteToParcel() const { return true; }
 
 void TypeNamespace::Init() {
-  Add(new ByteType());
-  Add(new PrimitiveType(
-      "int",
-      "cstdint", "int32_t", "readInt32", "writeInt32",
-      "readInt32Vector", "writeInt32Vector"));
-  Add(new PrimitiveType(
-      "long",
-      "cstdint", "int64_t", "readInt64", "writeInt64",
-      "readInt64Vector", "writeInt64Vector"));
-  Add(new PrimitiveType(
-      "float",
-      kNoHeader, "float", "readFloat", "writeFloat",
-      "readFloatVector", "writeFloatVector"));
-  Add(new PrimitiveType(
-      "double",
-      kNoHeader, "double", "readDouble", "writeDouble",
-      "readDoubleVector", "writeDoubleVector"));
-  Add(new PrimitiveType(
-      "boolean",
-      kNoHeader, "bool", "readBool", "writeBool",
-      "readBoolVector", "writeBoolVector"));
+  Add(std::make_unique<ByteType>());
+  Add(std::make_unique<PrimitiveType>("int", "cstdint", "int32_t", "readInt32", "writeInt32",
+                                      "readInt32Vector", "writeInt32Vector"));
+  Add(std::make_unique<PrimitiveType>("long", "cstdint", "int64_t", "readInt64", "writeInt64",
+                                      "readInt64Vector", "writeInt64Vector"));
+  Add(std::make_unique<PrimitiveType>("float", kNoHeader, "float", "readFloat", "writeFloat",
+                                      "readFloatVector", "writeFloatVector"));
+  Add(std::make_unique<PrimitiveType>("double", kNoHeader, "double", "readDouble", "writeDouble",
+                                      "readDoubleVector", "writeDoubleVector"));
+  Add(std::make_unique<PrimitiveType>("boolean", kNoHeader, "bool", "readBool", "writeBool",
+                                      "readBoolVector", "writeBoolVector"));
   // C++11 defines the char16_t type as a built in for Unicode characters.
-  Add(new PrimitiveType(
-      "char",
-      kNoHeader, "char16_t", "readChar", "writeChar",
-      "readCharVector", "writeCharVector"));
+  Add(std::make_unique<PrimitiveType>("char", kNoHeader, "char16_t", "readChar", "writeChar",
+                                      "readCharVector", "writeCharVector"));
 
   Type* string_array_type = new CppArrayType(
       ValidatableType::KIND_BUILT_IN, "java.lang", "String",
       "utils/String16.h", "::android::String16",
-      "readString16Vector", "writeString16Vector", false);
+      "::std::unique_ptr<::android::String16>", "readString16Vector",
+      "writeString16Vector", false);
 
   Type* nullable_string_type =
       new Type(ValidatableType::KIND_BUILT_IN, "java.lang", "String",
                {"memory", "utils/String16.h"}, "::std::unique_ptr<::android::String16>",
                "readString16", "writeString16");
 
-  string_type_ = new Type(ValidatableType::KIND_BUILT_IN, "java.lang", "String",
-                          {"utils/String16.h"}, "::android::String16",
-                          "readString16", "writeString16",
-                          string_array_type, nullable_string_type);
-  Add(string_type_);
+  AddAndSetMember(&string_type_,
+                  std::make_unique<Type>(ValidatableType::KIND_BUILT_IN, "java.lang", "String",
+                                         std::vector<std::string>{"utils/String16.h"},
+                                         "::android::String16", "readString16", "writeString16",
+                                         string_array_type, nullable_string_type));
 
   using ::android::aidl::kAidlReservedTypePackage;
   using ::android::aidl::kUtf8InCppStringClass;
@@ -462,68 +405,82 @@ void TypeNamespace::Init() {
   Type* cpp_utf8_string_array = new CppArrayType(
       ValidatableType::KIND_BUILT_IN,
       kAidlReservedTypePackage, kUtf8InCppStringClass,
-      "string", "::std::string",
+      "string", "::std::string", "::std::unique_ptr<::std::string>",
       "readUtf8VectorFromUtf16Vector", "writeUtf8VectorAsUtf16Vector",
       false);
-  Type* nullable_cpp_utf8_string_type = new Type(
-      ValidatableType::KIND_BUILT_IN,
-      kAidlReservedTypePackage, kUtf8InCppStringClass,
-      {"string", "memory"}, "::std::unique_ptr<::std::string>",
-      "readUtf8FromUtf16", "writeUtf8AsUtf16");
-  Add(new Type(
-      ValidatableType::KIND_BUILT_IN,
-      kAidlReservedTypePackage, kUtf8InCppStringClass,
-      {"string"}, "::std::string", "readUtf8FromUtf16", "writeUtf8AsUtf16",
-      cpp_utf8_string_array, nullable_cpp_utf8_string_type));
+  Type* nullable_cpp_utf8_string_type =
+      new Type(ValidatableType::KIND_BUILT_IN, kAidlReservedTypePackage, kUtf8InCppStringClass,
+               std::vector<std::string>{"string", "memory"}, "::std::unique_ptr<::std::string>",
+               "readUtf8FromUtf16", "writeUtf8AsUtf16");
+  Add(std::make_unique<Type>(ValidatableType::KIND_BUILT_IN, kAidlReservedTypePackage,
+                             kUtf8InCppStringClass, std::vector<std::string>{"string"},
+                             "::std::string", "readUtf8FromUtf16", "writeUtf8AsUtf16",
+                             cpp_utf8_string_array, nullable_cpp_utf8_string_type));
 
   Type* nullable_ibinder = new Type(
       ValidatableType::KIND_BUILT_IN, "android.os", "IBinder",
       {"binder/IBinder.h"}, "::android::sp<::android::IBinder>",
       "readNullableStrongBinder", "writeStrongBinder");
-  ibinder_type_ = new Type(
-      ValidatableType::KIND_BUILT_IN, "android.os", "IBinder",
-      {"binder/IBinder.h"}, "::android::sp<::android::IBinder>",
-      "readStrongBinder", "writeStrongBinder",
-      kNoArrayType, nullable_ibinder);
-  Add(ibinder_type_);
 
-  Add(new MapType());
+  AddAndSetMember(&ibinder_type_,
+                  std::make_unique<Type>(ValidatableType::KIND_BUILT_IN, "android.os", "IBinder",
+                                         std::vector<std::string>{"binder/IBinder.h"},
+                                         "::android::sp<::android::IBinder>", "readStrongBinder",
+                                         "writeStrongBinder", kNoArrayType, nullable_ibinder));
 
-  Add(new BinderListType());
-  Add(new StringListType());
-  Add(new Utf8InCppStringListType());
+  Add(std::make_unique<MapType>());
+
+  Add(std::make_unique<BinderListType>());
+  Add(std::make_unique<StringListType>());
+  Add(std::make_unique<Utf8InCppStringListType>());
 
   Type* fd_vector_type = new CppArrayType(
       ValidatableType::KIND_BUILT_IN, kNoPackage, "FileDescriptor",
       "android-base/unique_fd.h",
-      "::android::base::unique_fd",
+      "::android::base::unique_fd", "::android::base::unique_fd",
       "readUniqueFileDescriptorVector", "writeUniqueFileDescriptorVector",
       false);
 
-  Add(new Type(
-      ValidatableType::KIND_BUILT_IN, kNoPackage, "FileDescriptor",
-      {"android-base/unique_fd.h"}, "::android::base::unique_fd",
-      "readUniqueFileDescriptor", "writeUniqueFileDescriptor",
-      fd_vector_type));
+  Add(std::make_unique<Type>(ValidatableType::KIND_BUILT_IN, kNoPackage, "FileDescriptor",
+                             std::vector<std::string>{"android-base/unique_fd.h"},
+                             "::android::base::unique_fd", "readUniqueFileDescriptor",
+                             "writeUniqueFileDescriptor", fd_vector_type));
 
-  void_type_ = new class VoidType();
-  Add(void_type_);
+  Type* pfd_vector_type =
+      new CppArrayType(ValidatableType::KIND_BUILT_IN, "android.os", "ParcelFileDescriptor",
+                       "binder/ParcelFileDescriptor.h", "::android::os::ParcelFileDescriptor",
+                       "::android::os::ParcelFileDescriptor", "readParcelableVector",
+                       "writeParcelableVector", false);
+
+  Type* nullable_pfd_type =
+      new Type(ValidatableType::KIND_BUILT_IN, "android.os", "ParcelFileDescriptor",
+               std::vector<std::string>{"memory", "binder/ParcelFileDescriptor.h"},
+               "::std::unique_ptr<::android::os::ParcelFileDescriptor>", "readParcelable",
+               "writeNullableParcelable");
+
+  Add(std::make_unique<Type>(ValidatableType::KIND_BUILT_IN, "android.os", "ParcelFileDescriptor",
+                             std::vector<std::string>{"binder/ParcelFileDescriptor.h"},
+                             "::android::os::ParcelFileDescriptor", "readParcelable",
+                             "writeParcelable", pfd_vector_type, nullable_pfd_type));
+
+  // Qualify VoidType so we don't get collisions with the VoidType method
+  AddAndSetMember(&void_type_, std::make_unique<class VoidType>());
 }
 
-bool TypeNamespace::AddParcelableType(const AidlParcelable& p,
-                                      const string& filename) {
-  if (p.GetCppHeader().empty()) {
-    LOG(ERROR) << "Parcelable " << p.GetCanonicalName()
-               << " has no C++ header defined.";
+bool TypeNamespace::AddParcelableType(const AidlParcelable& p, const std::string& filename) {
+  const std::string cpp_header = p.AsStructuredParcelable() ? GetCppHeader(p) : p.GetCppHeader();
+
+  if (cpp_header.empty()) {
+    AIDL_ERROR(p) << "Parcelable " << p.GetCanonicalName() << " has no C++ header defined.";
     return false;
   }
-  Add(new ParcelableType(p, filename));
+
+  Add(std::make_unique<ParcelableType>(p, cpp_header, filename));
   return true;
 }
 
-bool TypeNamespace::AddBinderType(const AidlInterface& b,
-                                  const string& file_name) {
-  Add(new BinderType(b, file_name));
+bool TypeNamespace::AddBinderType(const AidlInterface& b, const std::string& filename) {
+  Add(std::make_unique<BinderType>(b, filename));
   return true;
 }
 
@@ -560,38 +517,9 @@ bool TypeNamespace::AddMapType(const std::string& /* key_type_name */,
   return false;
 }
 
-bool TypeNamespace::IsValidPackage(const string& package) const {
-  if (package.empty()) {
-    return false;
-  }
-
-  auto pieces = Split(package, ".");
-  for (const string& piece : pieces) {
-    if (is_cpp_keyword(piece)) {
-      return false;
-    }
-  }
-
-  return true;
-}
-
-const ValidatableType* TypeNamespace::GetArgType(const AidlArgument& a,
-    int arg_index,
-    const std::string& filename,
-    const AidlInterface& interface) const {
-  const string error_prefix = StringPrintf(
-      "In file %s line %d parameter %s (%d):\n    ",
-      filename.c_str(), a.GetLine(), a.GetName().c_str(), arg_index);
-
-  // check that the name doesn't match a keyword
-  if (is_cpp_keyword(a.GetName().c_str())) {
-    cerr << error_prefix << "Argument name is a C++ keyword"
-         << endl;
-    return nullptr;
-  }
-
-  return ::android::aidl::TypeNamespace::GetArgType(a, arg_index, filename,
-                                                    interface);
+const ValidatableType* TypeNamespace::GetArgType(const AidlArgument& a, int arg_index,
+                                                 const AidlDefinedType& context) const {
+  return ::android::aidl::TypeNamespace::GetArgType(a, arg_index, context);
 }
 
 }  // namespace cpp
